#include "angle.h"
using namespace std;
using namespace acp;

namespace acp {
  Angle::RandomBranch Angle::branch;
  ZeroAngle angle0;
}

int Angle::Compare::sign () {
  if (a == b || a->identical(b))
    return 0;
  if (a == &angle0)
    return -1;
  if (b == &angle0)
    return 1;
  PV2 aU = a->getU();
  PV2 bU = b->getU();
  if (aU.y.sign() > 0 && bU.y.sign() < 0)
    return -1;
  if (aU.y.sign() < 0 && bU.y.sign() > 0)
    return 1;
  return bU.cross(aU).sign();
}

int Angle::Circulation::sign () {
  static int count;
  if (++count == 0)
    cout << "this is it" << endl;
  if (a->identical(b) || b->identical(c))
    return 0;
  if (a->identical(c))
    cout << "this is it" << endl;
  if (a->getU().cross(c->getU()).sign() > 0) {
    if (a->getU().cross(b->getU()).sign() > 0 &&
        c->getU().cross(b->getU()).sign() < 0)
      return 1;
  }
  else {
    if (a->getU().cross(b->getU()).sign() > 0 ||
        c->getU().cross(b->getU()).sign() < 0)
      return 1;
  }
  return -1;
}

void ExplicitAngle::calculate () {
  PPoly2 pp2 = getPoly()->getPoly();
  Parameter a = pp2.a[1];
  Parameter b = pp2.a[2];
  Parameter c = pp2.a[0];
  Parameter a2pb2 = a*a + b*b;
  Parameter r = -c / a2pb2;
  PV2 p(a * r, b * r);
  Parameter s = (a2pb2 - c*c).sqrt() / a2pb2;
  PV2 q(-b * s, a * s);
  if (flag)
    set(p + q);
  else
    set(p - q);
}

vector<Angle*> AnglePoly::getExplicitAngles (PPoly2 &pp2) {
  vector<Angle*> angles;
  Parameter a = pp2.a[1];
  Parameter b = pp2.a[2];
  Parameter c = pp2.a[0];
  Parameter a2pb2 = a*a + b*b;
  Parameter d = a2pb2 - c*c;
  if (d > 0) {
    Parameter r = -c / a2pb2;
    PV2 p(a * r, b * r);
    Parameter s = d.sqrt() / a2pb2;
    PV2 q(-b * s, a * s);
    angles.push_back(new ExplicitAngle(p + q, true, this));
    angles.push_back(new ExplicitAngle(p - q, false, this));
  }
  return angles;
}

vector<Angle*> AnglePoly::getRoots () {
  bool failed = false;
  while (true)
    try {
      PPoly2 poly = getPoly();
      vector<Angle*> angles;
      if (poly.d == 1)
        angles = getExplicitAngles(poly);
      else {
        for (int flag = 0; flag <= 1; flag++) {
          vector<Parameter> roots = getRoots(poly, flag);
          for (int i = 0; i < roots.size(); i++)
            angles.push_back(new ImplicitAngle(roots[i], flag, this));
        }
      }
        
      for (int i = 0; i < angles.size(); i++)
        dynamic_cast<RootAngle*>(angles[i])->index = i;
    
      for (int i = 1; i < angles.size(); i++) {
        Angle *angle = angles[i];
        int j = i;
        while (j > 0 && Angle::Compare(angles[j-1], angle).unsafe() > 0) {
          angles[j] = angles[j-1];
          j--;
        }
        angles[j] = angle;
      }
    
      for (int i = 0; i < angles.size(); i++) {
        dynamic_cast<RootAngle*>(angles[i])->index = i;
	if (failed) {
          if (poly.d == 1)
            dynamic_cast<ExplicitAngle*>(angles[i])->decreasePrecision();
          else
            dynamic_cast<ImplicitAngle*>(angles[i])->decreasePrecision();
        }
      }
    
      if (failed)
        BaseObject::decreaseAll();

      return angles;
    } catch (SignException se) {
      if (Parameter::highPrecision < 212)
        Parameter::highPrecision = 212;
      failed = true;
    }
}

vector<Parameter> AnglePoly::getRoots (PPoly2 &poly, bool flag) {
  PPoly1 g = univariate(poly, flag);
  PV2 b = Angle::getBranch();
  return g.roots(b.x, b.y);
}

Parameter AnglePoly::polish (Parameter s, bool flag) {
  PPoly2 poly = getPoly();
  PPoly1 g = univariate(poly, flag);
  return g.polish(s);
}

PPoly1 AnglePoly::univariate (PPoly2 &poly, bool flag) {
  int d = poly.degree(), dg = 0;
  vector<Parameter> a(2*d+1);
  for (int i = 0; i < 2*d + 1; ++i)
    a[i] = Parameter::constant(0);
  for (int i = 0; i < poly.nt; ++i) {
    int c = poly.m[2*i], s = poly.m[2*i+1];
    term(poly.a[i], c, s, d, flag, a);
    dg = max(dg, 2*d - s);
  }
  return PPoly1(dg, &a[0]);
}

int choose (int k, int n)
{
  int res = 1;
  for (int i = 0; i < k; ++i)
    res *= n - i;
  for (int i = 2; i <= k; ++i)
    res /= i;
  return res;
}

int power (int b, int d)
{
  int p = 1;
  for (int i = 0; i < d; ++i)
    p *= b;
  return p;
}

void AnglePoly::term (const Parameter &u, int c, int s, int d, bool flag,
                      vector<Parameter> &a) {
  int e = d - c - s;
  for (int k = 0; k <= c + e; ++k) {
    int l = 0;
    for (int i = 0; i <= k; ++i)
      l += power(-1, i)*choose(i, c)*choose(k - i, e);
    l *= flag || (c+s)%2 == 0 ? power(2, s) : - power(2, s);
    int p = 2*k + s;
    a[p] = a[p] + u*l;
  }
}

int RootAngle::Sign::sign () {
  PPoly2 ppoly2 = poly->getPoly();
  PV2 u = root->getU();
  Parameter x[] = { u.x, u.y };
  return ppoly2.value(x).sign();
}

#ifdef BLEEN
PPoly2 Unitizer::getPoly () {
  PPoly2 poly(2);
  poly.m[0] = 1; poly.m[1] = 0;
  poly.m[2] = 0; poly.m[3] = 1;
  poly.setDegree();
  PV2 xy = v->getV();
  poly.a[0] = -xy.y;
  poly.a[1] = xy.x;
  return poly;
}

int Unitizer::Dot::sign () {
  return v->getV().dot(r->getU()).sign();
}

Angle *Unitizer::unitize (VectorObject *v) {
  Unitizer *u = new Unitizer(v);
  vector<Angle*> roots = u->getRoots();
  if (Unitizer::Dot(v, roots[0]) > 0)
    return roots[0];
  else
    return roots[1];
}

Angle *MeanAngle::mean (Angle *u, Angle *v, double t) {
  MeanAngle *m = new MeanAngle(u, v, t);
  return Unitizer::unitize(m);
}
#endif
