/*
  ACP (Adaptive Controlled Precision) Library
  for robust computational geometry

  Copyright (c) 2013-07-15
  Victor Joseph Milenkovic
  University of Miami
  vjm@miami.edu
  Elisha Sacks
  Purdue University
  eps@cs.purdue.edu

  This file contains code described in

Robust Complete Path Planning in the Plane
Victor Milenkovic, Elisha Sacks, and Steven Trac
Proceedings of the Workshop on the Algorithmic Foundations of Robotics (WAFR)
pages 37-52, 2012

   This code is under development.
   It is free for use for academic and research purposes.
   Permission from the authors is required for commercial use.
*/

#include "poly.h"
using namespace std;
using namespace acp;

vector<Root*> Poly::getRoots (Parameter lo, Parameter hi) {
  bool failed = false;
  while (true)
    try {
      PPoly1 pp1 = getPoly();
      vector<Parameter> proots = pp1.roots(lo, hi);
      vector<Root*> roots(proots.size());

      for (int i = 0; i < roots.size(); i++)
        roots[i] = new Root(this, i, proots[i]);

      if (failed) {
        for (int i = 0; i < roots.size(); i++)
          roots[i]->decreasePrecision();
        BaseObject::decreaseAll();
      }

      return roots;
    } catch (SignException se) {
      if (Parameter::highPrecision < 212)
        Parameter::highPrecision = 212;
      lo.increasePrecision();
      hi.increasePrecision();
      failed = true;
    }
}

Parameter Poly::polish (Parameter r) {
  PPoly1 g = getPoly();
  return g.polish(r);
}
