/*
  ACP (Adaptive Controlled Precision) Library
  for robust computational geometry

  Copyright (c) 2013-07-15
  Victor Joseph Milenkovic
  University of Miami
  vjm@miami.edu
  Elisha Sacks
  Purdue University
  eps@cs.purdue.edu

  This file contains code described in

Robust Complete Path Planning in the Plane
Victor Milenkovic, Elisha Sacks, and Steven Trac
Proceedings of the Workshop on the Algorithmic Foundations of Robotics (WAFR)
pages 37-52, 2012

   This code is under development.
   It is free for use for academic and research purposes.
   Permission from the authors is required for commercial use.
*/

#include "object.h"
#include "pv.h"
#include "ppoly.h"

namespace acp {

class RefCount {
  int refCount;
public:
  RefCount () : refCount(0) {}
  virtual ~RefCount () {}
  void incRef () { refCount++; }
  void decRef () { if (--refCount == 0) delete this; }
};

class Root;

class Poly : public RefCount {
public:
  Poly () {}
  virtual PPoly1 getPoly () = 0;
  std::vector<Root*> getRoots (Parameter lo, Parameter hi);
  Parameter polish (Parameter s);
};

class Root : public Object<Parameter> {
  friend class Poly;
  Poly *poly;
  int index;
public:
  Root (Poly *poly, int index, Parameter &r) : poly(poly), index(index) {
    set(r);
    poly->incRef();
  }
  ~Root () { poly->decRef(); } 
  Poly *getPoly () { return poly; }
  int getIndex () { return index; }
  void calculate () { set(poly->polish(get())); }
};

}

