#include "poly.h"
using namespace std;
using namespace acp;

// Pure parameter class.
// Object template can only allow pure parameter classes.
struct Parameter4 {
  Parameter p[4];
};

// Quartic polynomial with with four given inputs as roots.
class Quartic : public Object<Parameter4>, public Poly {
public:
  Quartic (double d[4]) {
    Parameter4 p4;
    for (int i = 0; i < 4; i++)
      p4.p[i] = Parameter::input(d[i]);
    // Creating input parameters perturbs the input values.
    set(p4);
  }

  // Parameters are inputs, so no calculation is necessary.
  void calculate () {}

  // PPoly1 is Parameter polynomial in one variable (ppoly.h)
  // represented by its coefficients.
  PPoly1 getPoly () {
    Parameter4 p4 = get();
    Parameter *r = p4.p;	// input roots
    Parameter c[5];		// coefficients of polynomial
    c[0] = r[0] * r[1] * r[2] * r[3];
    c[1] = -(r[1] * r[2] * r[3] +
             r[0] * r[2] * r[3] +
             r[0] * r[1] * r[3] +
             r[0] * r[1] * r[2]);
    c[2] = (r[0] * r[1] + r[0] * r[2] + r[0] * r[3] +
            r[1] * r[2] + r[1] * r[3] + r[2] * r[3]);
    c[3] = -(r[0] + r[1] + r[2] + r[3]);
    c[4] = Parameter::constant(1);
    // Monic polynomial so leading coefficient is EXACTLY 1, not
    // approximately 1.  (::constant does NOT perturb.)
    return PPoly1(4, c);
  }
};
  
int main (int argc, char *argv[]) {
  // Seed for random number generator.
  if (argc >= 2) {
    unsigned seed = atoi(argv[1]);
    cout << "seed " << seed << endl;
    srandom(seed);
  }

  double d[] = { 1, 2, 3, 4 };
  if (argc >= 6) {
    for (int i = 0; i < 4; i++)
      d[i] = atof(argv[2 + i]);
  }

  // Set the correct rounding mode.
  Parameter::enable();

  Parameter lo = Parameter::input(0);
  Parameter hi = Parameter::input(5);
  Quartic q(d);
  vector<Root*> r = q.getRoots(lo, hi);
  for (int i = 0; i < r.size(); i++)
    cout << d[i] << " "
         << r[i]->get().lb()-d[i] << " "
	 << r[i]->get().ub()-d[i] << " "
	 << r[i]->get().ub()-r[i]->get().lb() << endl;

/*
./poly_test 1 1 2 3 4
seed 1
1 8.43701e-09 8.43702e-09 1.06582e-14
2 1.33413e-08 1.33414e-08 8.57093e-14
3 2.45361e-08 2.45362e-08 1.66534e-13
4 -2.25342e-08 -2.25341e-08 9.23706e-14
*/
// Input roots are perturbed as part of ACP.

/*
./poly_test 1 1 2 2 2
seed 1
1 8.43702e-09 8.43702e-09 4.4409e-16
2 -1.35205e-08 -1.35205e-08 4.4409e-16
2 1.33414e-08 1.33414e-08 8.88179e-16
2 1.84021e-08 1.84f021e-08 8.88179e-16
*/
// Because of nearly identical roots, higher precision has to be used
// to ensure separate root intervals.
}

    
