#ifndef PPOLY
#define PPOLY

#include "acp.h"
#include <vector>

namespace acp {

class PPoly1 {
public:
  PPoly1 (int d, Parameter *a = 0);
  PPoly1 (const PPoly1 &f);
  ~PPoly1 () {delete [] a; }
  PPoly1 & operator= (const PPoly1 &f);
  bool increased () const { return a[0].increased(); }
  bool decreased () const { return a[0].decreased(); }
  Parameter value (const Parameter &x) const;
  Parameter der (const Parameter &x) const;
  PPoly1 der () const;

  Parameter polish (const Parameter &x) const;
  Parameter shrinkBracket (Parameter x, int sub) const;
  Parameter shrinkBracketB (const Parameter &x) const;
  std::vector<Parameter> roots (Parameter lb, Parameter ub) const;
  std::vector<Parameter> roots1 (Parameter lb, Parameter ub) const;
  std::vector<Parameter> roots2 (Parameter lb, Parameter ub) const;

  int d;
  Parameter *a;
};

class PPoly2 {
 public:
  PPoly2 () : nt(0), a(0), m(0) {}
  PPoly2 (int nt) : nt(nt), a(new Parameter [nt]), m(new int [2*nt]) {}
  PPoly2 (int nt, Parameter *a, int *m);
  PPoly2 (const PPoly2 &p);
  void setDegree ();
  ~PPoly2 () {delete [] a; delete [] m; }
  PPoly2 & operator= (const PPoly2 &p);
  int degree () const { return d; }
  bool increased () const { return a[0].increased(); }
  bool decreased () const { return a[0].decreased(); }
  Parameter value (Parameter *x) const;

  PPoly2 operator* (const PPoly2 &g) const;
  PPoly2 plusCtimes (double c, const PPoly2 &b) const;
  PPoly2 operator+ (const PPoly2 &b) const { return plusCtimes(1.0, b); }
  PPoly2 operator- (const PPoly2 &b) const { return plusCtimes(-1.0, b); }

  int nt, d;
  Parameter *a;
  int *m;
};

}
#endif
