#include "predicates3d.h"
#include <iostream>
using std::cout;
using std::endl;

int main (int argc, char *argv[]) {
  // Seed for random number generator.
  if (argc == 2) {
    unsigned seed = atoi(argv[1]);
    cout << "seed " << seed << endl;
    srandom(seed);
  }

  double zero = 0.0123456789123456789;
  double one = 0.9876543210987654321;

  InputPoint a(zero, zero, zero);
  InputPoint b(one, zero, zero);
  InputPoint c(zero, one, zero);
  InputPoint d(zero, zero, one);
  InputPoint e(one, one, zero);

  // Set the correct rounding mode.  Rounding is towards positive
  // infinity for implementing interval arithmetic.
  Parameter::enable();

  if (Orient3D(&a, &b, &c, &d) > 0)
    cout << "abcd positive" << endl;
  else
    cout << "abcd negative" << endl;

  // Even though e is numerically in the same plane as a, b, and c,
  // ACP employs perturbation, so the orientation is non-zero.  Try
  // running the program with different seeds.  (Don't give up: seeds
  // 1 through 8 give the same value, by strange coincidence.)
  cout << "abce " << Orient3D(&a, &b, &c, &e) << endl;

  // This is the plane of triangle bcd.
  TrianglePlane bcd(&b, &c, &d);
  
  // This point is the projection of point a onto plane bcd.
  PlanePoint f(&bcd, &a);

  cout << "b precision " << b.getP().x.precision() << endl;

  // Be careful with these projection points.  They can lead to other
  // coplanarities.
  // f lies in the same plane as b, c, and d by construction, no
  // matter how you perturb them.  So Orient3D(f, b, c, d) is
  // identically zero.  This will throw a PrecisionException because
  // ACP will run past the maximum allowed precision:
  // Parameter::maxPrecision.  This is set to 848 bits by default.
  try {
    cout << "fbcd " << Orient3D(&f, &b, &c, &d) << endl;
  } catch (PrecisionException e) {
    cout << "fbcd zero (exception)" << endl;
  }

  cout << "b precision " << b.getP().x.precision() << endl;

  BaseObject::decreaseAll();

  cout << "b precision " << b.getP().x.precision() << endl;

  if (Orient3D(&a, &b, &c, &d) > 0)
    cout << "abcd positive" << endl;
  else
    cout << "abcd negative" << endl;

  Parameter::disable();
}

