#include "predicates3d.h"
#include <iostream>
// using std::cout;
// using std::endl;

int PlaneSide::sign () {
  PV3 v = plane->getV();
  Parameter d = plane->getD();
  PV3 p = point->getP();
  return (v.dot(p) - d).sign();
}

int Orient3D::sign () {
  /*
  PV3 dp = d->getP();
  PV3 da = a->getP() - dp;
  PV3 db = b->getP() - dp;
  PV3 dc = c->getP() - dp;
  Parameter o = da.cross(db).dot(dc);
  */
  PV3 dp = d->getP();
  PV3 da = a->getP();
  PV3 db = b->getP();
  PV3 dc = c->getP();

  Parameter o = (da.cross(db).dot(dc) -
		 dp.cross(db).dot(dc) +
		 dp.cross(da).dot(dc) -
		 dp.cross(da).dot(db));

  return o.sign();
}

int InSphere::sign () {
  PV3 ep = e->getP();
  PV3 ea = a->getP() - ep;
  PV3 eb = b->getP() - ep;
  PV3 ec = c->getP() - ep;
  PV3 ed = d->getP() - ep;

  return (ea.cross(eb).dot(ec) * ed.dot(ed) -
          eb.cross(ec).dot(ed) * ea.dot(ea) +
          ec.cross(ed).dot(ea) * eb.dot(eb) -
          ed.cross(ea).dot(eb) * ec.dot(ec)).sign();
}

void TrianglePlane::calculate () {
  PV3 ap = a->getP();
  PV3 bp = b->getP();
  PV3 cp = c->getP();
  PV3 v = (bp - ap).cross(cp - ap);
  Parameter d = v.dot(ap);
  // set(PlaneEquation(v, v.dot(ap)));
  Object<PV3>::set(v);
  Object<Parameter>::set(d);
}
  
void PlanePoint::calculate ()  {
  PV3 v = plane->getV();
  Parameter d = plane->getD();
  PV3 p = point->getP();
  set(p - v * ((v.dot(p) - d) / v.dot(v)));
}

void IntersectionPoint::calculate () {
  PV3 t = tail->getP();
  PV3 h = head->getP();
  PV3 v = plane->getV();
  Parameter d = plane->getD();
  // (t + s (h - t)) * v = d
  // t * v + s (h - t) * v = d
  // s = (d - t * v) / ((h - t) * v)

  Parameter s = (d - t.dot(v)) / (h - t).dot(v);
  set(t + (h - t) * s);
}

void IntersectionPosition::calculate () {
  PV3 t = tail->getP();
  PV3 h = head->getP();
  PV3 p = getP();
  Object<Parameter>::set(p.dot(h - t));
}

    
    
